/*

  AUDIO SWITCHER

  Based on a serial device demonstration from the LUFA project
  (www.lufa-lib.org).  LUFA is Copyright (C) Dean Camera, 2012.

  Changes made to the serial device demonstration program are
  Copyright (C) Tim Marston, 2012.

  For more information:
    * see README, or
    * visit http://ed.am/

*/

/*
  Copyright 2012  Dean Camera (dean [at] fourwalledcubicle [dot] com)

  Permission to use, copy, modify, distribute, and sell this
  software and its documentation for any purpose is hereby granted
  without fee, provided that the above copyright notice appear in
  all copies and that both that the copyright notice and this
  permission notice and warranty disclaimer appear in supporting
  documentation, and that the name of the author not be used in
  advertising or publicity pertaining to distribution of the
  software without specific, written prior permission.

  The author disclaim all warranties with regard to this
  software, including all implied warranties of merchantability
  and fitness.  In no event shall the author be liable for any
  special, indirect or consequential damages or any damages
  whatsoever resulting from loss of use, data or profits, whether
  in an action of contract, negligence or other tortious action,
  arising out of or in connection with the use or performance of
  this software.
*/

/** \file
 *
 *  Main source file for the VirtualSerial demo. This file contains the main
 *  tasks of the demo and is responsible for the initial application hardware
 *  configuration.
 */

#include "VirtualSerial.h"
#include "wiring.h"

/** LUFA CDC Class driver interface configuration and state information. This
 *  structure is passed to all CDC Class driver functions, so that multiple
 *  instances of the same class within a device can be differentiated from one
 *  another.
 */
USB_ClassInfo_CDC_Device_t VirtualSerial_CDC_Interface =
	{
		.Config =
			{
				.ControlInterfaceNumber   = 0,
				.DataINEndpoint           =
					{
						.Address          = CDC_TX_EPADDR,
						.Size             = CDC_TXRX_EPSIZE,
						.Banks            = 1,
					},
				.DataOUTEndpoint =
					{
						.Address          = CDC_RX_EPADDR,
						.Size             = CDC_TXRX_EPSIZE,
						.Banks            = 1,
					},
				.NotificationEndpoint =
					{
						.Address          = CDC_NOTIFICATION_EPADDR,
						.Size             = CDC_NOTIFICATION_EPSIZE,
						.Banks            = 1,
					},
			},
	};

/** Standard file stream for the CDC interface when set up, so that the virtual
 *  CDC COM port can be used like any regular character stream in the C APIs
 */
static FILE USBSerialStream;


#define BUTTON 0b10000000
#define LEDB 0b00100000
#define RELAY 0b00000010

void init()
{
	// set PD7 (button) to input and enable pull-up
	DDRD &= ~BUTTON;
	PORTD |= BUTTON;

	// set PD5 (ledb) to output
	DDRD |= LEDB;

	// set PB1 (relay) to output
	DDRB |= RELAY;
}

void loop()
{
	// debounce button (PD7)
	static int button_state = 0;
	static int saved_button_state = 0;
	static int saved_since = 0;
	unsigned long now = millis();
	unsigned long real_button_state = PIND & BUTTON;
	if( saved_button_state != real_button_state ) {
		saved_button_state = real_button_state;
		saved_since = now;
	}
	else if( saved_button_state != button_state &&
			 now - saved_since > 25 )
	{
		button_state = saved_button_state;
	}

	// on rising edge of press, set toggle
	static int last_button_state = 0;
	bool toggle = false;
	if( last_button_state != button_state )
	{
		last_button_state = button_state;
		if( !button_state ) toggle = true;
	}


	// toggle the relay
	static bool relay_state = false;
	if( toggle )
	{
		relay_state = !relay_state;

		// switch relay
		if( relay_state )
			PORTB |= RELAY;
		else
			PORTB &= ~RELAY;

		// switch LED
		if( relay_state )
			PORTD &= ~LEDB;
		else
			PORTD |= LEDB;
	}
}


/** Main program entry point. This routine contains the overall program flow,
 *  including initial setup of all components and the main program loop.
 */
int main(void)
{
	SetupHardware();

	/* Create a regular character stream for the interface so that it can be
	 * used with the stdio.h functions */
	CDC_Device_CreateStream(&VirtualSerial_CDC_Interface, &USBSerialStream);

	LEDs_SetAllLEDs(LEDMASK_USB_NOTREADY);
	sei();

	for (;;)
	{
		loop();

		/* Must throw away unused bytes from the host, or it will lock up while
		 * waiting for the device */
		CDC_Device_ReceiveByte(&VirtualSerial_CDC_Interface);

		CDC_Device_USBTask(&VirtualSerial_CDC_Interface);
		USB_USBTask();
	}
}

/** Configures the board hardware and chip peripherals for the demo's
 * functionality. */
void SetupHardware(void)
{
	/* Disable watchdog if enabled by bootloader/fuses */
	MCUSR &= ~(1 << WDRF);
	wdt_disable();

	/* Disable clock division */
	clock_prescale_set(clock_div_1);

	/* Hardware Initialization */
	LEDs_Init();
	USB_Init();

	// do my init
	init();

	// set up arduino wiring.c
	init_wiring_c();
}

/** Event handler for the library USB Connection event. */
void EVENT_USB_Device_Connect(void)
{
	LEDs_SetAllLEDs(LEDMASK_USB_ENUMERATING);
}

/** Event handler for the library USB Disconnection event. */
void EVENT_USB_Device_Disconnect(void)
{
	LEDs_SetAllLEDs(LEDMASK_USB_NOTREADY);
}

/** Event handler for the library USB Configuration Changed event. */
void EVENT_USB_Device_ConfigurationChanged(void)
{
	bool ConfigSuccess = true;

	ConfigSuccess &= CDC_Device_ConfigureEndpoints(&VirtualSerial_CDC_Interface);

	LEDs_SetAllLEDs(ConfigSuccess ? LEDMASK_USB_READY : LEDMASK_USB_ERROR);
}

/** Event handler for the library USB Control Request reception event. */
void EVENT_USB_Device_ControlRequest(void)
{
	CDC_Device_ProcessControlRequest(&VirtualSerial_CDC_Interface);
}

